#!/bin/bash
#
# Copyright 2007, Dave Markus (Davem at Mich Dot Com)
#   If you fix bugs, or make improvements, I would
#   appreciate seeing a copy. I am sure there is much room for improvement :)
#   Be sure to put "KDar_BurnSlice" in your subject line so I get your message.
#
# This script is free software, and you are welcome to redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 (draft-2) of the License, 
# or (at your option) any later version. http://gplv3.fsf.org/
#
# Release:
# 2007.02.05 alpha - Dave Markus, first release
#   Tested with:
#   - Ubuntu 6.06 (dapper)
#   - Kdar version 2.0.6, (C) 2005 Johnathan K Burchill
#   - dar version 2.2.4, Copyright (C) 2002 Denis Corbin
#   - par2cmdline version 0.4, Copyright (C) 2003 Peter Brian Clements
#   - growisofs version 5.21 by <appro@fy.chalmers.se>
# 2007.02.06 beta - Dave Markus
#   - Fixed problem with zipping catalog from multi-slice archive. 
#     Catalog file should be named same as the first slice file
# 2007.02.07 beta - Dave Markus
#   - Fixed problem with cleanup of zipped catalog file. 
# 2007.02.13 beta - Dave Markus
#   - Fixed problem with cleanup of zipped catalog file (again). 
#
SCRIPT_NAME="KDar_BurnSlice "           #Prefix used to identify the source of the messages generated by this script
SCRIPT_VER="2007.02.13 beta"            #Release number
#
echo "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-="
echo $SCRIPT_NAME " 1-\""$1"\" 2-\""$2"\" 3-\""$3"\" 4-\""$4"\" 5-\""$5"\" 6-\""$6"\""
if [ -z "$1" ] ||  [ -z "$2" ] || [ -z "$3" ] || [ -z "$4" ] || [ $1 == "-h" ] || [ $1 == "-help" ] || [ $1 == "--help" ]; then
  echo $SCRIPT_NAME "- Copyright 2006, Dave Markus, GPL v3 (http://gplv3.fsf.org/)"
  echo "Release: "$SCRIPT_VER"        Davem at Mich Dot Com "
  echo "         Script used by KDar to burn slices to the DVD drive. "
  echo "         This script can add data redundancy using parity (a par2 file), "
  echo "         by making a backup copy of the slice file or both. "
  echo "         It can also verify and retry burning the slice if verify fails."
  echo "         It also extracts DAR catalog for use in future differential backups." 
  echo $SCRIPT_NAME "- Usage:" $SCRIPT_NAME "1 2 3 4 [5] [6] "
  echo "       1 - Path to file "
  echo "       2 - Base File Name "
  echo "       3 - Slice Number "
  echo "       4 - File Extension "
  echo "       5 - -vN (Verify No) -vY (verify Yes) -vR (verify and re-burn - default) "
  echo "       6 - -rN (redundancy No) -rP (using a parity file, Smallest - default) "
  echo "           -rC (redundant copy of file, Fastest) -rB (Both parity and copy) "
  echo "           * Returns 0 if burn and verify are successful."
  echo "* To use with KDar, save this file to ~/Scripts/KDar_BurnSlice.sh then use " 
  echo "  KDar-Option group-Slicing-CommandToRunAfterWritingEachSlice and enter: "
  echo "  ~/Scripts/KDar_BurnSlice.sh %p %b %n %e >> %p/%b.log 2>&1 "
  echo "* When using Kdar to export a dar script (for automatic use with cron),  "
  echo "  use KDar-Archive-Create-ArchiveName to add: _$\"(date +%y%m%d-%H%M)\" "
  echo "  to the end of the archive name to make files on the DVD unique for each run. "
  echo "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-="
  exit 255
fi

VERIFY=""                               #Default to verify and retry
REDUNDANT=""                            #Default to redundant via parity file
RESULT=255                              #Running result
MRESULT=255                             #Media mount result
DDEV="/dev/hdc"                         #The destination device 
MDEV="/media/cdrom0"                    #Path to the mounted media 
APP_NAME="KDar_BurnSlice_growisofs"     #Used to identify the source of DVDs burned by this script
DARSTATIC="/usr/bin/dar_static"         #Stand-alone copy of Dar, used to restore backups 
DARCP="/usr/bin/dar_cp"                 #Dar_cp, used to copy damaged backup slices from defective media 
DARRESCUE="/usr/bin/dares-qt"           #Dar Rescue, qt based GUI used to restore damaged backups from defective media 
PAR="/usr/bin/par2"                     #par2, used to fix damaged backup slice files 
TIME=$(date +%y.%m.%d" "%T)             #Date and time
VOL_ID=$2_$(date +%y%m%d)               #Used to name the CDs or DVDs burned by this script
FILE=$1/$2.$3.$4                        #Full path to the file that will be added to the DVD
BASE=$1/$2                              #The path and base name of the slice, used to isolate the catalog
CATDIR=$1/Cat                           #Path to the catalog files
CATBASE=$CATDIR/$2                      #The catalog file that we will save as the basis for later diff backups
CATFILE=$CATBASE.1.$4                   #The catalog file that we will save as the basis for later diff backups (catalog file name is always slice 1)
CATFILEZ=""                             #The zipped catalog file that we will burn 
DFILE=$MDEV/$2.$3.$4                    #Full path to the slice file that we just burned to the DVD
PFILE=$1/$2.$3.$4.par2                  #Full path to the parity file
BFILESET=$1/$2.$3.$4.*                  #glob points to the full parity and backup file set
BFILE=""                                #Full path to the backup copy of the file that we will burn to the DVD
PFILES=""                               #Glob points to par2 files
DBFILE=$DFILE.bak                       #Full path to the backup copy of the file that we just burned to the DVD
LOG=$1/$2.log                           #Full path to the log file
LOGZ=""                                 #Full path to the compressed log file
N_RETRY=3                               #Number of times we will try to burn the file
N_RETRYCOUNT=1                          #Number of times we have tried to burn the file

echo "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-="
#echo "TIME= "$TIME
#echo "VOL_ID= "$VOL_ID
#echo "FILE= "$FILE
#echo "BASE= "$BASE
#echo "CATDIR= "$CATDIR
#echo "CATBASE= "$CATBASE
#echo "DDIR= "$DDIR
#echo "DFILE= "$DFILE
#echo "PFILE= "$PFILE
#echo "BFILESET= "$BFILESET
#echo "BFILE= "$BFILE
#echo "DBFILE= "$DBFILE
#echo "LOG= "$LOG
#exit 255

if ! [ -z "$5" ]; then
  RESULT=253
  if [ $5 == "-vN" ]; then
    VERIFY_DESC="-vN Do not verify after burning slice, Not Reccomended!"
    VERIFY=$5
    RESULT=0
  elif  [ $5 == "-vY" ]; then
    VERIFY_DESC="-vY Verify slice after burning"
    VERIFY=$5
    RESULT=0
  elif  [ $5 == "-vR" ]; then
    VERIFY_DESC="-vR Verify slice after burning and re-burn if failed"
    VERIFY=$5
    RESULT=0
  elif [ $5 == "-rN" ]; then
    REDUNDANT_DESC="-rN No slice file redundancy."
    REDUNDANT=$5
    RESULT=0
  elif [ $5 == "-rP" ]; then
    REDUNDANT_DESC="-rP Use a parity file for slice file redundancy"
    REDUNDANT=$5
    RESULT=0
  elif [ $5 == "-rC" ]; then
    REDUNDANT_DESC="-rC Use a second copy of the slice file for redundancy"
    REDUNDANT=$5
    RESULT=0
  elif [ $5 == "-rB" ]; then
    REDUNDANT_DESC="-rB Use both parity and a second copy of the slice for redundancy"
    REDUNDANT=$5
    RESULT=0
  elif ! [ -z "$6" ]; then
    RESULT=253
    if [ $6 == "-vN" ]; then
      VERIFY_DESC="-vN Do not verify after burning slice, Not Reccomended!"
      VERIFY=$6
      RESULT=0
    elif  [ $6 == "-vY" ]; then
      VERIFY_DESC="-vY Verify slice after burning"
      VERIFY=$6
      RESULT=0
    elif  [ $6 == "-vR" ]; then
      VERIFY_DESC="-vR Verify slice after burning and re-burn if failed"
      VERIFY=$6
      RESULT=0
    elif [ $6 == "-rN" ]; then
      REDUNDANT_DESC="-rN No slice file redundancy."
      REDUNDANT=$6
      RESULT=0
    elif [ $6 == "-rP" ]; then
      REDUNDANT_DESC="-rP Use a parity file for slice file redundancy"
      REDUNDANT=$6
      RESULT=0
    elif [ $6 == "-rC" ]; then
      REDUNDANT_DESC="-rC Use a second copy of the slice file for redundancy"
      REDUNDANT=$6
      RESULT=0
    elif [ $6 == "-rB" ]; then
      REDUNDANT_DESC="-rB Use both parity and a second copy of the slice for redundancy"
      REDUNDANT=$6
      RESULT=0
    fi #Check 6
  fi #Check 5
fi #Have 5

if [ -z $VERIFY ]; then
  VERIFY="-vR"                            #Default to verify and retry
  VERIFY_DESC="-vR Verify slice after burning and re-burn if failed"
fi
if [ -z $REDUNDANT ]; then
  REDUNDANT="-rP"                         #Default to redundant via parity file
  REDUNDANT_DESC="-rP Use a parity file for slice file redundancy"
fi

echo $SCRIPT_NAME "Time:             " $TIME
echo $SCRIPT_NAME "Version:          " $SCRIPT_VER
echo $SCRIPT_NAME "Path to file:     " $1
echo $SCRIPT_NAME "Base File Name:   " $2
echo $SCRIPT_NAME "Slice Number:     " $3
echo $SCRIPT_NAME "File Extension:   " $4
echo $SCRIPT_NAME "Verify:           " $VERIFY_DESC
echo $SCRIPT_NAME "Redundancy:       " $REDUNDANT_DESC

if [ $RESULT == 253 ]; then
  echo $SCRIPT_NAME "Invalid Parameter: "$5" "$6" use -vN, -vY, -vR, -rN, -rP, -rC or -rB"
  echo "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-="
  exit $RESULT
fi

if ! [ -e $FILE ]; then
  echo $SCRIPT_NAME $(date +%T" ") "- Unable to locate slice file: " $FILE
  echo "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-="
  exit 252
fi

if ! [ -d $CATDIR ]; then
  echo  $SCRIPT_NAME "- Making new catalog directory: " $CATDIR
  mkdir $CATDIR
fi

#Current version of Dar can not test slices, maybe someday...
#echo $SCRIPT_NAME "- Testing the slice: " $FILE
#dar -t $FILE
#RESULT=$?
#
#if [ $RESULT == 0 ]; then                                               #If the slice is bad, exit with fail result
#  echo $SCRIPT_NAME $(date +%T" ") "- * Bad slice: " $FILE " !No need to continue! * "
#  echo "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-="
#  exit 251
#fi

# Isolate The Catalog
if [ -e $CATFILE ]; then                                        
  echo $SCRIPT_NAME $(date +%T" ") "- Deleting the old Catalog file: "$CATFILE
  rm $CATFILE
fi
if [ -e $FILE.gz ]; then                                        
  echo $SCRIPT_NAME $(date +%T" ") "- Deleting the old Catalog gz file: "$FILE.gz
  rm $FILE.gz
fi
echo $SCRIPT_NAME $(date +%T" ") "- Isolating the Catalog "
dar -C $CATBASE -A $BASE -Q -z9 > /dev/null 2>&1                         #See if we can isolate a catalog from this slice
CATRESULT=$?
if [ $CATRESULT == 0 ]; then                                             #If we isolated a catalog, zip it and add it to the burn job
  echo $SCRIPT_NAME $(date +%T" ") "- Compressing the Catalog File"
  CATFILEZ=$BASE.1.$4.gz
  gzip -9 $CATFILE -c -c > $CATFILEZ
  if ! [ -e $CATFILEZ ]; then                                            #Do not try to burn the zip file if it was not created
    CATFILEZ=""
    echo $SCRIPT_NAME $(date +%T" ") "- Unable to compress the Catalog File! "
  fi
else
  echo $SCRIPT_NAME $(date +%T" ") "- This slice contains no Catalog Data! "
fi

# Clean up old files, save good par files if needed (because testing is faster than re-creating them)
if [ $REDUNDANT == "-rP" ] || [ $REDUNDANT == "-rC" ] || [ $REDUNDANT == "-rB" ]; then
  ls $BFILESET > /dev/null 2>&1
  RESULT=$?
  if [ $RESULT == 0 ]; then                                                #If the backup files(s) already exist, see if the par files are useable
    if [ -e $PFILE ]; then                                                 #If the par files already exist
      echo $SCRIPT_NAME $(date +%T" ") "- Testing Existing Par Files"     
      par2 v -q -q $FILE                                                   #Test to see if the par files are still good
      RESULT=$?
      if [ $RESULT == 0 ]; then                                            #Save the par files if OK  (because testing is faster than re-creating them)
        echo $SCRIPT_NAME $(date +%T" ") "- Existing Par Files Are Good"    
        PFILES=$FILE".*.par2 "$FILE.par2 
        if [ -e $1/$2.$3.$4.bak ]; then                                    #If the par files are good, delete the bak file
          echo $SCRIPT_NAME $(date +%T" ") "- Delete old Backup File"
          rm $1/$2.$3.$4.bak
        fi
      else                                                                 #If the par files are bad, delete all redundant files
        echo $SCRIPT_NAME $(date +%T" ") "- Delete old Backup and Par Files"
        rm $BFILESET
      fi
    fi
  fi
else                                                                       #If no redundancy required
  ls $BFILESET > /dev/null 2>&1
  RESULT=$?
  if [ $RESULT == 0 ]; then                                                #delete all redundant files
    echo $SCRIPT_NAME $(date +%T" ") "- Delete old Backup and Par Files"
    rm $BFILESET
  fi
fi

# Make backup file
if [ $REDUNDANT == "-rC" ] || [ $REDUNDANT == "-rB" ]; then
  echo $SCRIPT_NAME $(date +%T" ") "- Making Backup File"
  cp $FILE $1/$2.$3.$4.bak
  RESULT=$?
  if [ $RESULT == 0 ] && [ $VERIFY != "-vN" ]; then
    echo $SCRIPT_NAME $(date +%T" ") "- Testing Backup File"
    cmp $FILE $1/$2.$3.$4.bak
    RESULT=$?
  fi
  if [ $RESULT == 0 ]; then
    BFILE=$1/$2.$3.$4.bak
  else
    echo $SCRIPT_NAME $(date +%T" ") "- *** Copy Error: "$RESULT" Unable To Make Backup File ***"
    echo "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-="
    exit $RESULT
  fi
fi

# Make Par FIle
if [ $REDUNDANT == "-rP" ] || [ $REDUNDANT == "-rB" ]; then
  if ! [ -e $PFILE ]; then                                              #If the PAR2 files do not already exist
    echo $SCRIPT_NAME $(date +%T" ") "- Making Parity File Set"
    par2 c -q -q -r10 -n1 $FILE                                         #Make new ones
    RESULT=$?
    if [ $RESULT == 0 ] && [ $VERIFY != "-vN" ]; then
      echo $SCRIPT_NAME $(date +%T" ") "- Verify Parity File Set"
      par2 v -q -q $FILE                                                #Verify par files (catch hardware errors)
      RESULT=$?
    fi
    if [ $RESULT == 0 ]; then
      echo $SCRIPT_NAME $(date +%T" ") "- Done Making Parity File Set"
      PFILES=$FILE".*.par2 "$FILE.par2 
    else
      echo $SCRIPT_NAME $(date +%T" ") "- *** Par2 Result: "$RESULT" Unable To Make Parity File Set ***"
      echo "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-="
      exit $RESULT
    fi
  fi
fi

# Loop Burns The Files To the DVD
while [ $N_RETRYCOUNT -le $N_RETRY ]; do                            #Try burning N_RETRY times
  echo "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
#Update the compressed log file
  echo $SCRIPT_NAME $(date +%T" ") "- Burn Loop: Attempt " $N_RETRYCOUNT " of " $N_RETRY
  if [ -e $LOG ]; then                                              #If we can find the log, compress it
    echo $SCRIPT_NAME $(date +%T" ") "- Compressing the log file"
    LOGZ=$1/$2.log.gz
    gzip -9 $LOG -c -c > $LOGZ
    if ! [ -e $LOGZ ]; then                                         #Do not try to burn the zip file if it was not created
      LOGZ=""
    fi
  fi
#Burn the files to an existing file system
  if [ $N_RETRYCOUNT -eq 1 ]; then
    echo $SCRIPT_NAME $(date +%T" ") "- Burning: "$FILE $BFILE $PFILES $CATFILEZ $LOGZ
  else
    echo $SCRIPT_NAME $(date +%T" ") "- Retry Burning: "$FILE $BFILE $PFILES $CATFILEZ $LOGZ
  fi
  growisofs -quiet -use-the-force-luke=tty -use-the-force-luke=notray -M $DDEV -r -J -T $FILE $BFILE $PFILES $CATFILEZ $LOGZ
  RESULT=$?
  echo $SCRIPT_NAME $(date +%T" ") "- growisofs, RESULT is: " $RESULT
  if [ $RESULT == 156 ]; then                                                     #If disk is full, give up
    echo $SCRIPT_NAME $(date +%T" ") "- Media is full! "
    break
  fi
  if [ $RESULT == 252 ] || [ $RESULT == 157 ] || [ $RESULT == 150 ]; then         #If there is no file system, burn a new one and include a copy of dar_static, dar_cp and par (used for restoring files)
    if [ $N_RETRYCOUNT -eq 1 ]; then                                              #Only erase the old file system on the first try so as not to risk deleteing previously written slices 
#Update the compressed log file
      if [ -e $LOG ]; then                                                        #If we can find the log file, compress it
        echo $SCRIPT_NAME $(date +%T" ") "- Compressing the log file"
        LOGZ=$1/$2.log.gz
        gzip -9 $LOG -c > $LOGZ
        if ! [ -e $LOGZ ]; then                                                   #Do not try to burn the log file if it was not created
          LOGZ=""
        fi
      fi
#Burn the files to a new file system
      echo $SCRIPT_NAME $(date +%T" ") "- Erase Old File System and Retry Burning: "$DARRESCUE $DARSTATIC $DARCP $PAR $FILE $BFILE $PFILES $CATFILEZ $LOGZ
      growisofs -quiet -use-the-force-luke=tty -use-the-force-luke=notray -Z $DDEV -V "${VOL_ID}" -A "${APP_NAME}" -r -J -T $DARRESCUE $DARSTATIC $DARCP $PAR $FILE $BFILE $PFILES $CATFILEZ $LOGZ 
      RESULT=$?
      echo $SCRIPT_NAME $(date +%T" ") "- growisofs, RESULT is: " $RESULT
      if [ $RESULT == 156 ]; then                                                 #If disk is full, give up
        echo $SCRIPT_NAME $(date +%T" ") "- Media is full! "
        break
      fi
    fi
  fi
  if [ $RESULT != 0 ]; then
    echo $SCRIPT_NAME $(date +%T" ") "- *** growisofs, Error while burning! ***"  #If burn failed, log error
    if [ -e $CATFILE ]; then                                                      #No need to save the catalog
      rm $CATFILE
    fi
  else
#Mount the destination media
    echo $SCRIPT_NAME $(date +%T" ") "- Mounting " $DDEV " as " $MDEV
    mount $MDEV
    MRESULT=$?
    if [ $RESULT != 0 ]; then                                             #if mount fails, un-mount and re-try 
      umount $DDEV
      mount $MDEV
      MRESULT=$?
    fi
    if [ $MRESULT != 0 ]; then                                            #if mount fails again, skip verify, log a warning and continue
      echo $SCRIPT_NAME $(date +%T" ") "- *** Error "$RESULT" mounting "$DDEV" as "$MDEV" ***"
      echo $SCRIPT_NAME "-   User may not have mount privleges "
      echo $SCRIPT_NAME "-   Edit /etc/fstab and make sure that there is an entry for the CD/DVD " 
      echo $SCRIPT_NAME "-   device entry that ends with: " $MDEV " auto ro,noauto,user,exec 0 0 "
      echo $SCRIPT_NAME "- *** Unable to mount media - skipping file verification! *** "
    else
      if [ $VERIFY == "-vN" ]; then                                       #Do we skip the verify/retry?
        echo $SCRIPT_NAME $(date +%T" ") "- Skipping the Verify Step - is faster but not reccomended!"
        break 
      else                                                                #If the burn and mount was a success, compare the files
#Test the backup slice file that we just burned
        if [ $REDUNDANT == "-rC" ] || [ $REDUNDANT == "-rB" ]; then       #Compare the backup file
          echo $SCRIPT_NAME $(date +%T" ") "- Verify Backup Copy"                     
          cmp $DBFILE $FILE 
          RESULT=$?          
          if [ $RESULT == 0 ]; then                                    
            echo $SCRIPT_NAME $(date +%T" ") "- Backup Copy Is Good"                     
          else  
            echo $SCRIPT_NAME $(date +%T" ") "- *** Compare " $DBFILE" and "$FILE" Failed! ***"
            touch -m $FILE $BFILE $PFILES                                  #Change the inode so growisofs will re-burn the files
          fi #backup good?
        fi #chk backup?
#Test the slice file that we just burned
        if [ $RESULT == 0 ]; then                                          #Compare the primary file
          echo $SCRIPT_NAME $(date +%T" ") "- Verify Primary Copy "$DFILE                     
          cmp $DFILE $FILE
          RESULT=$?   
          if [ $RESULT == 0 ]; then                                    
            echo $SCRIPT_NAME $(date +%T" ") "- Primary Copy Is Good"                     
          else
            echo $SCRIPT_NAME $(date +%T" ") "- *** Compare " $DFILE" and "$FILE" Failed! ***"
            touch -m $FILE $BFILE $PFILES                                  #Change the inode so growisofs will re-burn the files
          fi #primary good?
        fi #chk primary?
#Test the parity files that we just burned
        if [ $RESULT == 0 ]; then                                    
          if [ $REDUNDANT == "-rP" ] || [ $REDUNDANT == "-rB" ]; then      #Check the parity?
            echo $SCRIPT_NAME $(date +%T" ") "- Verify Parity for "$DFILE
            par2 v -q -q $DFILE
            RESULT=$?
            if [ $RESULT == 0 ]; then 
              echo $SCRIPT_NAME $(date +%T" ") "- Parity Files are Good"                     
            else                                   
              touch -m $FILE $BFILE $PFILES                                #Change the inode so growisofs will re-burn the files
              echo $SCRIPT_NAME $(date +%T" ") "- *** Verify Parity Failed! ***"
            fi #parity good?
          fi #chk parity?
        fi #pri good?
      fi #verify?
    fi #mounted?
  fi #growisofs good?
#Clean up 
  if [ $RESULT == 0 ]; then                                                          #If we passed verify tests
    if [ $CATRESULT == 0 ] || [ $3 != "1" ]; then                                    #Clean up and exit but dont delete the first slice unless it held the catalog
      echo $SCRIPT_NAME $(date +%T" ") "- Deleting: "$FILE" "$BFILESET" "$LOGZ
      rm $FILE $BFILESET $LOGZ 
      if [ $CATRESULT == 0 ]; then
        if [ -e $BASE.1.dar ]; then
          echo $SCRIPT_NAME $(date +%T" ") "- Deleting: "$BASE.1.dar
          rm $BASE.1.dar
        fi
        if [ -e $CATFILEZ ]; then                                                    #No need to save the zipped catalog
          echo $SCRIPT_NAME $(date +%T" ") "- Deleting: "$CATFILEZ
          rm $CATFILEZ
        fi
      fi 
    else
      echo $SCRIPT_NAME $(date +%T" ") "- Deleting: "$BFILESET" "$LOGZ
      rm $BFILESET $LOGZ
    fi
    echo $SCRIPT_NAME $(date +%T" ") "- Burn Slice Completed! "
    break
  else                                                                                 #If verify failed
    echo $SCRIPT_NAME $(date +%T" ") "- *** Burn Slice " $FILE " failed " $N_RETRYCOUNT " times, Result = " $RESULT " ***"
    let N_RETRYCOUNT=N_RETRYCOUNT+1                                                    #Increment the retry counter
  fi
done
#Clean up 
if [ $RESULT != 0 ]; then                                                            #If we failed to burn the slice
  if [ -e $CATFILE ]; then                                                           #No need to save the catalog
    echo $SCRIPT_NAME $(date +%T" ") "- Deleting: "$CATFILE
    rm $CATFILE
  fi
  if [ -e $CATFILEZ ]; then                                                          #No need to save the zipped catalog
    echo $SCRIPT_NAME $(date +%T" ") "- Deleting: "$CATFILEZ
    rm $CATFILEZ
  fi
fi
echo "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
# Mount the destination media before exiting 
if ! [ $MRESULT == 0 ]; then                                                           #If not already mounted, mount the media 
  echo $SCRIPT_NAME $(date +%T" ") "- Mounting "$DDEV" as "$MDEV
  mount $MDEV
  MRESULT=$?
  if [ $MRESULT == 0 ]; then                                                           #If mount fails, un-mount and re-try 
    echo $SCRIPT_NAME $(date +%T" ") "- "$DDEV" Successfully mounted as "$MDEV
  else
    echo $SCRIPT_NAME $(date +%T" ") "- Try unmounting "$DDEV" First"
    umount $DDEV
    echo $SCRIPT_NAME $(date +%T" ") "- Try Mounting "$DDEV" as "$MDEV" Again"
    mount $MDEV
    MRESULT=$?
    if [ $MRESULT == 0 ]; then                                                         #if mount fails twice, give up 
      echo $SCRIPT_NAME $(date +%T" ") "- "$DDEV" Successfully mounted as "$MDEV
    else
      echo $SCRIPT_NAME $(date +%T" ") "- *** Error "$MRESULT" unable to mount "$DDEV" as "$MDEV" ***"
    fi
  fi
fi
#exit
echo $SCRIPT_NAME $(date +%T" ") "- Burn Slice exiting with return code: "$RESULT
#echo "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-="
exit $RESULT
#
# If you would prefer not to use this script, here is a
# simple command for KDar that works some of the time:
#
#   growisofs -M /dev/dvd -r -J -T %p/%b.%n.%e
#
# It has none of the formatting, verify, parity, cataloging or 
# redundancy as is provided by this script.
#
